@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 
 ## system-config-printer
 
@@ -19,7 +19,7 @@
 ## along with this program; if not, write to the Free Software
 ## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-import pycurl, urllib, platform, threading, tempfile, traceback
+import requests, urllib.request, urllib.parse, urllib.error, platform, threading, tempfile, traceback
 import os, sys
 from xml.etree.ElementTree import XML
 from . import Device
@@ -43,7 +43,7 @@ class _QueryThread (threading.Thread):
         self.parameters = parameters
         self.callback = callback
         self.user_data = user_data
-        self.result = ""
+        self.result = b''
 
         self.setDaemon (True)
         _debugprint ("+%s" % self)
@@ -53,42 +53,31 @@ class _QueryThread (threading.Thread):
 
     def run (self):
 
-        # Callback function for pycURL collecting the data coming from
-        # the web server
-        def collect_data(result):
-            self.result += result;
-            return len(result)
-
         # CGI script to be executed
         query_command = "/query.cgi"
         # Headers for the post request
         headers = {"Content-type": "application/x-www-form-urlencoded",
                    "Accept": "text/plain"}
         params = ("%s&uilanguage=%s&locale=%s" %
-                  (urllib.urlencode (self.parameters),
+                  (urllib.parse.urlencode (self.parameters),
                    self.parent.language[0],
                    self.parent.language[0]))
         self.url = "https://%s%s?%s" % (self.parent.base_url, query_command, params)
         # Send request
         result = None
-        self.result = ""
+        self.result = b''
         status = 1
         try:
-            curl = pycurl.Curl()
-            curl.setopt(pycurl.SSL_VERIFYPEER, 1)
-            curl.setopt(pycurl.SSL_VERIFYHOST, 2)
-            curl.setopt(pycurl.WRITEFUNCTION, collect_data)
-            curl.setopt(pycurl.URL, self.url)
-            status = curl.perform()
-            if status == None: status = 0
-            if (status != 0):
-                self.result = sys.exc_info ()
+            req = requests.get(self.url, verify=True)
+            self.result = req.content
+            status = 0
         except:
             self.result = sys.exc_info ()
-            if status == None: status = 0
+            if status is None:
+                status = 0
 
         _debugprint ("%s: query complete" % self)
-        if self.callback != None:
+        if self.callback is not None:
             self.callback (status, self.user_data, self.result)
 
 class OpenPrinting:
@@ -98,7 +87,7 @@ class OpenPrinting:
         locale.setlocale().
         @type language: string
         """
-        if language == None:
+        if language is None:
             import locale
             try:
                 language = locale.getlocale(locale.LC_MESSAGES)
@@ -177,7 +166,7 @@ class OpenPrinting:
                     id = printer.find ("id")
                     make = printer.find ("make")
                     model = printer.find ("model")
-                    if id != None and make != None and model != None:
+                    if id is not None and make is not None and model is not None:
                         idtxt = id.text
                         maketxt = make.text
                         modeltxt = model.text
@@ -278,24 +267,37 @@ class OpenPrinting:
 
                 for driver in root.findall ('driver'):
                     id = driver.attrib.get ('id')
-                    if id == None:
+                    if id is None:
                         continue
 
                     dict = {}
                     for attribute in ['name', 'url', 'supplier', 'license',
                                       'shortdescription' ]:
                         element = driver.find (attribute)
-                        if element != None and element.text != None:
+                        if element is not None and element.text is not None:
                             dict[attribute] = _normalize_space (element.text)
 
                     element = driver.find ('licensetext')
-                    if element != None and element.text != None:
+                    if element is not None and element.text is not None:
                         dict['licensetext'] = element.text
+                    if not 'licensetext' in dict or \
+                       dict['licensetext'] is None:
+                        element = driver.find ('licenselink')
+                        if element is not None:
+                            license_url = element.text
+                            if license_url is not None:
+                                try:
+                                    req = requests.get(license_url, verify=True)
+                                    dict['licensetext'] = \
+                                        req.content.decode("utf-8")
+                                except:
+                                    _debugprint('Cannot retrieve %s' %
+                                                license_url)
 
                     for boolean in ['nonfreesoftware', 'recommended',
                                     'patents', 'thirdpartysupplied',
                                     'manufacturersupplied']:
-                        dict[boolean] = driver.find (boolean) != None
+                        dict[boolean] = driver.find (boolean) is not None
 
                     # Make a 'freesoftware' tag for compatibility with
                     # how the OpenPrinting API used to work (see trac
@@ -304,10 +306,10 @@ class OpenPrinting:
 
                     supportcontacts = []
                     container = driver.find ('supportcontacts')
-                    if container != None:
+                    if container is not None:
                         for sc in container.findall ('supportcontact'):
                             supportcontact = {}
-                            if sc.text != None:
+                            if sc.text is not None:
                                 supportcontact['name'] = \
                                     _normalize_space (sc.text)
                             else:
@@ -319,23 +321,23 @@ class OpenPrinting:
                     if supportcontacts:
                         dict['supportcontacts'] = supportcontacts
 
-                    if not dict.has_key ('name') or not dict.has_key ('url'):
+                    if 'name' not in dict or 'url' not in dict:
                         continue
 
                     container = driver.find ('functionality')
-                    if container != None:
+                    if container is not None:
                         functionality = {}
                         for attribute in ['text', 'lineart', 'graphics',
                                           'photo', 'speed']:
                             element = container.find (attribute)
-                            if element != None:
+                            if element is not None:
                                 functionality[attribute] = element.text
                         if functionality:
                             dict[container.tag] = functionality
 
                     packages = {}
                     container = driver.find ('packages')
-                    if container != None:
+                    if container is not None:
                         for arch in container.getchildren ():
                             rpms = {}
                             for package in arch.findall ('package'):
@@ -344,11 +346,11 @@ class OpenPrinting:
                                                   'release', 'url', 'pkgsys',
                                                   'fingerprint']:
                                     element = package.find (attribute)
-                                    if element != None:
+                                    if element is not None:
                                         rpm[attribute] = element.text
 
                                 repositories = package.find ('repositories')
-                                if repositories != None:
+                                if repositories is not None:
                                     for pkgsys in repositories.getchildren ():
                                         rpm.setdefault('repositories', {})[pkgsys.tag] = pkgsys.text
 
@@ -360,7 +362,7 @@ class OpenPrinting:
 
                     ppds = []
                     container = driver.find ('ppds')
-                    if container != None:
+                    if container is not None:
                         for each in container.getchildren ():
                             ppds.append (each.text)
 
@@ -376,11 +378,19 @@ class OpenPrinting:
         if isinstance(model, Device):
             model = model.id
 
+        architecture = platform.machine()
+
+        # On Intel, we could be running a 32bit user space with a 64bit kernel, in
+        # which case platform.machine() will return x86_64, leading to downloading
+        # the wrong printer driver, so we make sure we ask for i386 in that case.
+        if architecture == 'x86_64' and platform.architecture()[0] == '32bit':
+            architecture = 'i386'
+
         params = { 'type': 'drivers',
                    'moreinfo': '1',
                    'showprinterid': '1',
                    'onlynewestdriverpackages': '1',
-                   'architectures': platform.machine(),
+                   'architectures': architecture,
                    'noobsoletes': '1',
                    'onlyfree': str (self.onlyfree),
                    'onlymanufacturer': str (self.onlymanufacturer),
@@ -455,7 +465,7 @@ def _simple_gui ():
         def query_callback (self, status, user_data, result):
             Gdk.threads_enter ()
             self.tv.get_buffer ().set_text (str (result))
-            file ("result.xml", "w").write (str (result))
+            open ("result.xml", "w").write (str (result))
             Gdk.threads_leave ()
 
     q = QueryApp()
