@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 
 ## Copyright (C) 2008, 2009, 2010, 2012, 2014 Red Hat, Inc.
 ## Authors:
@@ -27,7 +27,7 @@ import subprocess
 import threading
 import config
 import gettext
-gettext.install(domain=config.PACKAGE, localedir=config.localedir, unicode=True)
+gettext.install(domain=config.PACKAGE, localedir=config.localedir)
 from debug import *
 
 # Initialise threading for D-Bus.  This is needed as long as it is
@@ -57,8 +57,9 @@ class TimedSubprocess(Timed):
         self.parent = parent
         self.show_dialog = show_dialog
         for f in [self.subp.stdout, self.subp.stderr]:
-            if f != None:
+            if f is not None:
                 source = GLib.io_add_watch (f,
+                                            GLib.PRIORITY_DEFAULT,
                                             GLib.IO_IN |
                                             GLib.IO_HUP |
                                             GLib.IO_ERR,
@@ -82,7 +83,7 @@ class TimedSubprocess(Timed):
             GLib.source_remove (self.wait_source)
         for source in self.io_source:
             GLib.source_remove (source)
-        if self.wait_window != None:
+        if self.wait_window is not None:
             self.wait_window.destroy ()
         return (self.output.get (self.subp.stdout, '').split ('\n'),
                 self.output.get (self.subp.stderr, '').split ('\n'),
@@ -96,7 +97,7 @@ class TimedSubprocess(Timed):
     def watcher (self, source, condition):
         if condition & GLib.IO_IN:
             buffer = self.output.get (source, '')
-            buffer += source.read ()
+            buffer += (source.read ()).decode("utf-8")
             self.output[source] = buffer
 
         if condition & GLib.IO_HUP:
@@ -178,7 +179,7 @@ class TimedOperation(Timed):
                                        kwargs=kwargs)
         self.thread.start ()
 
-        self.use_callback = callback != None
+        self.use_callback = callback is not None
         if self.use_callback:
             self.timeout_source = GLib.timeout_add (50, self._check_thread)
 
@@ -218,8 +219,8 @@ class TimedOperation(Timed):
         # Thread has finished.  Stop the sub-loop or trigger callback.
         self.timeout_source = False
         if self.use_callback:
-            if self.callback != None:
-                if self.context != None:
+            if self.callback is not None:
+                if self.context is not None:
                     self.callback (self.thread.result, self.thread.exception,
                                    self.context)
                 else:
