@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 
 ## Printing troubleshooter
 
@@ -21,7 +21,7 @@
 
 import cups
 from timedops import TimedOperation
-from base import *
+from .base import *
 class CheckLocalServerPublishing(Question):
     def __init__ (self, troubleshooter):
         Question.__init__ (self, troubleshooter, "Is local server publishing?")
@@ -47,7 +47,7 @@ class CheckLocalServerPublishing(Question):
             if len (printers) == 0:
                 return False
 
-            for name, printer in printers.iteritems ():
+            for name, printer in printers.items ():
                 if printer.get ('printer-is-shared', False):
                     break
 
